/* Special definitions for sharutils, processed by autoheader.
   Copyright (C) 1994 Free Software Foundation, Inc.
   François Pinard <pinard@iro.umontreal.ca>, 1994.
   Ulrich Drepper <drepper@gnu.ai.mit.edu>, 1995.
*/

/* Define if the current hostname may be found in /etc/systemid.  */
#undef HAVE_ETC_SYSTEMID

/* Define to the name of the distribution.  */
#undef PACKAGE

/* Define to 1 if ANSI function prototypes are usable.  */
#undef PROTOTYPES

/* Define to the version of the distribution.  */
#undef VERSION

/* Define to 1 for better use of the debugging malloc library.  See
   site ftp.antaire.com in antaire/src, file dmalloc/dmalloc.tar.gz.  */
#undef WITH_DMALLOC

/* Define if your locale.h file contains LC_MESSAGES.  */
#undef HAVE_LC_MESSAGES

/* Define to 1 if NLS is requested.  */
#undef ENABLE_NLS

/* Define as 1 if you have catgets and don't want to use GNU gettext.  */
#undef HAVE_CATGETS

/* Define as 1 if you have gettext and don't want to use GNU gettext.  */
#undef HAVE_GETTEXT

/* Define as 1 if you have the stpcpy function.  */
#undef HAVE_STPCPY
